'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelTypes = require('babel-types');

var _babelLiteralToAst = require('babel-literal-to-ast');

var _babelLiteralToAst2 = _interopRequireDefault(_babelLiteralToAst);

var _graphqlTag = require('graphql-tag');

var _graphqlTag2 = _interopRequireDefault(_graphqlTag);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug2.default)('babel-plugin-graphql-tag');

exports.default = () => {
  const compile = path => {
    const source = path.node.quasis.reduce((head, quasi) => {
      return head + quasi.value.raw;
    }, '');

    const expressions = path.get('expressions');

    expressions.forEach(expr => {
      if (!(0, _babelTypes.isIdentifier)(expr) && !(0, _babelTypes.isMemberExpression)(expr)) {
        throw expr.buildCodeFrameError('Only identifiers or member expressions are allowed by this plugin as an interpolation in a graphql template literal.');
      }
    });

    debug('compiling a GraphQL query', source);

    const queryDocument = (0, _graphqlTag2.default)(source);

    // If a document contains only one operation, that operation may be unnamed:
    // https://facebook.github.io/graphql/#sec-Language.Query-Document
    if (queryDocument.definitions.length > 1) {
      for (const definition of queryDocument.definitions) {
        if (!definition.name) {
          throw new Error('GraphQL query must have name.');
        }
      }
    }

    const body = (0, _babelLiteralToAst2.default)(queryDocument);

    if (expressions.length) {
      const definitionsProperty = body.properties.find(property => {
        return property.key.value === 'definitions';
      });

      const definitionsArray = definitionsProperty.value;

      const extraDefinitions = expressions.map(expr => {
        return (0, _babelTypes.memberExpression)(expr.node, (0, _babelTypes.identifier)('definitions'));
      });

      definitionsProperty.value = (0, _babelTypes.callExpression)((0, _babelTypes.memberExpression)(definitionsArray, (0, _babelTypes.identifier)('concat')), extraDefinitions);
    }

    debug('created a static representation', body);

    return body;
  };

  return {
    visitor: {
      ImportDeclaration(path) {
        if (path.node.source.value === 'graphql-tag') {
          path.remove();
        }
      },
      TaggedTemplateExpression(path) {
        if ((0, _babelTypes.isIdentifier)(path.node.tag, {
          name: 'gql'
        })) {
          try {
            debug('quasi', path.node.quasi);

            const body = compile(path.get('quasi'));

            path.replaceWith(body);
          } catch (error) {
            // eslint-disable-next-line no-console
            console.error('error', error);
          }
        }
      }
    }
  };
};
//# sourceMappingURL=index.js.map